<?php

declare(strict_types=1);

namespace Drupal\conditions_field\Plugin\Field\FieldWidget;

use Drupal\Core\Field\WidgetBase;
use Drupal\Component\Utility\Html;
use Drupal\Component\Serialization\Json;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Component\Plugin\PluginManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Defines the 'conditions_groups' field widget.
 * 
 * @todo new plugin in separate field-set
 *
 * @FieldWidget(
 *   id = "conditions_groups",
 *   label = @Translation("Condition groups (EXPERIMENTAL)"),
 *   field_types = {"conditions"},
 * )
 */
class ConditionsGroupsWidget extends WidgetBase {

  /**
   * Constructs a ConditionsGroupsWidget object.
   *
   * @param string $plugin_id
   *   The plugin_id for the widget.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field_definition
   *   The definition of the field to which the widget is associated.
   * @param array $settings
   *   The widget settings.
   * @param array $third_party_settings
   *   Any third party settings.
   * @param \Drupal\dct\DctCellTypePluginManagerInterface $cellTypeManager
   *   The cell type manager
   */
  public function __construct(
    $plugin_id,
    $plugin_definition,
    FieldDefinitionInterface $field_definition,
    array $settings,
    array $third_party_settings,
    protected PluginManagerInterface $cellTypeManager,
  ) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $third_party_settings);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['third_party_settings'],
      $container->get('plugin.manager.condition')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function form(FieldItemListInterface $items, array &$form, FormStateInterface $form_state, $get_delta = NULL) {
    parent::form($items, $form, $form_state, $get_delta);
    $parents = $form['#parents'];

    $field_name = $this->fieldDefinition->getName();
    $title = $this->fieldDefinition->getLabel();
    $description = $this->getFilteredDescription();
    $field_widget_complete_form = [
      // Aid in theming of widgets by rendering a classified container.
      // @todo: User container when in paragraphs.
      '#type' => $this->fieldDefinition->getTargetEntityTypeId() == 'paragraph' ? 'container' : 'fieldset',
      '#title' => $title,
      '#description' => $description,
      '#description_display' => 'before',
      '#tree' => TRUE,
      '#attributes' => [
        'class' => [
          'field--type-' . Html::getClass($this->fieldDefinition->getType()),
          'field--name-' . Html::getClass($field_name),
          'field--widget-' . Html::getClass($this->getPluginId()),
          'form-item',
        ],
      ],
      '#field_parents' => $parents,
      '#parents' => array_merge($parents, [$field_name])
    ];

    $field_widget_complete_form['condition_groups'] = [
      '#type' => 'fieldset',
      '#prefix' => '<div id="condition-groups-wrapper">',
      '#suffix' => '</div>',
    ];

    $field_widget_complete_form['condition_groups'] = [
      '#type' => 'conditions_groups',
      '#default_value' => $this->defaultValue($items),
    ];

    return $field_widget_complete_form;
  }

  /**
   * Get the default value for the condition groups form element.
   *
   * @param FieldItemListInterface $items
   *   The field value items.
   * @return array
   *   The default value.
   */
  protected function defaultValue(FieldItemListInterface $items): array {
    $default_value = [
      'group_condition_logic' => $items[0] ? $items[0]->getOperator() : 'and',
    ];
    foreach ($items->getValue() as $value) {
      if (empty($value['value'])) {
        continue;
      }
      $conditions = Json::decode($value['value']);
      $value = [
        'condition_logic' => $value['condition_logic'],
      ];
      foreach ($conditions as $uuid => $condition) {
        $value['conditions'][$uuid] = $condition;
      }
      $default_value[] = $value;
    }

    return $default_value;
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $field_widget_complete_form, array &$form, FormStateInterface $form_state) {
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function massageFormValues(array $values, array $form, FormStateInterface $form_state) {
    $massaged_values = parent::massageFormValues($values, $form, $form_state);
    return $massaged_values;
  }

  /**
   * {@inheritdoc}
   */
  public function extractFormValues(FieldItemListInterface $items, array $form, FormStateInterface $form_state) {
    $field_name = $this->fieldDefinition->getName();

    // Extract the values from $form_state->getValues().
    $path = array_merge($form['#parents'], [$field_name]);
    $copy_form_state = clone $form_state;
    $copy_form_state->cleanValues();
    $values = NestedArray::getValue($copy_form_state->getValues(), $path);
    if (!empty($values)) {
      $item_values = [];
      foreach ($values['condition_groups'] as $id => $condition_group) {
        if (!is_array($condition_group) || empty($condition_group['conditions'])) {
          continue;
        }
        $item_values[$id] = [
          'condition_logic' => $condition_group['condition_logic'],
          'group_condition_logic' => $values['condition_groups']['group_condition_logic'],
          'value' => JSON::encode($condition_group['conditions']),
        ];
      }
      $items->setValue($item_values);
    }

    return $values;
  }

}
