<?php

declare(strict_types=1);

namespace Drupal\conditions_field\Plugin\Field\FieldWidget;

use Drupal\Component\Serialization\Json;
use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;

/**
 * Defines the 'conditions' field widget.
 *
 * @todo Discuss this with Dominik.
 *
 * @todo Rebuild the widget so we get data for storing in the following format:
 *    delta = Condition Group
 *    value = conditions: {}
 *    group_condition_logic = operator for the conditions of the delta
 *    condition_logic = Overall operator between the deltas
 *
 * @todo Make sure that empty conditions are not returned in the value callback.
 *
 * @FieldWidget(
 *   id = "conditions",
 *   label = @Translation("Conditions"),
 *   field_types = {"conditions"},
 * )
 */
class ConditionsWidget extends WidgetBase {

  /**
   * {@inheritdoc}
   */
  public function form(FieldItemListInterface $items, array &$form, FormStateInterface $form_state, $get_delta = NULL) {
    parent::form($items, $form, $form_state, $get_delta);
    $parents = $form['#parents'];

    $conditions = JSON::decode($items->value ?? '[]');

    $field_name = $this->fieldDefinition->getName();
    $element = [
      '#type' => 'plugin_config',
      '#title' => t('Conditions'),
      '#description' => t('The conditions under which the @type should be displayed', [
        '@type' => $items->getParent()->getValue()->bundle(),
      ]),
      '#default_value' => $conditions,
      '#tree' => TRUE,
      '#weight' => 100,
      '#plugin_manager' => 'plugin.manager.condition',
      '#field_parents' => $parents,
      '#parents' => array_merge($parents, [$field_name]),
    ];

    $element['condition_logic'][0] = [
      '#type' => 'select',
      '#title' => new TranslatableMarkup('Condition logic'),
      '#required' => TRUE,
      '#options' => ['and' => 'and', 'or' => 'or'],
      '#default_value' => $items->condition_logic ?? 'and',
    ];

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function extractFormValues(FieldItemListInterface $items, array $form, FormStateInterface $form_state) {
    parent::extractFormValues($items, $form, $form_state);
    $field_name = $this->fieldDefinition->getName();

    // Extract the values from $form_state->getValues().
    $path = array_merge($form['#parents'], [$field_name]);
    $key_exists = NULL;
    $values = NestedArray::getValue($form_state->getUserInput(), $path, $key_exists);
    if (!empty($values)) {
      unset($values['plugin_group']);
      unset($values['actions']);
      $condition_logic = $values['condition_logic'];
      unset($values['condition_logic']);

      // @todo Find out why we have the empty value.
      $condition_configurations = [];
      foreach ($values as $uuid => $condition_configuration) {
        if (isset($condition_configuration['id'])) {
          $condition_configurations[$uuid] = $condition_configuration;
        }
      }
      $items->setValue([
        'condition_logic' => $condition_logic[0],
        'group_condition_logic' => 'or',
        'value' => JSON::encode($condition_configurations),
      ]);

      $form_state->setValue($field_name, $values);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function massageFormValues(array $values, array $form, FormStateInterface $form_state) {
    $field_name = $this->fieldDefinition->getName();
    $new_values = [];
    foreach ($values as $delta => $value) {
      $new_values[$value['_original_delta']] = $value;
      unset($new_values[$value['_original_delta']]['_original_delta']);
    }
    $form_state->setValue($field_name, $new_values);

    return $values;
  }

}
