<?php

namespace Drupal\conditions\Element;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Render\Attribute\FormElement;
use Drupal\Core\Render\Element\FormElementBase;
use Drupal\plugin_config_form_builder\Element\PluginConfig;

/**
 * Provides a conditions_groups element.
 */
#[FormElement('conditions_groups')]
class ConditionsGroups extends FormElementBase {

  /**
   * {@inheritdoc}
   */
  public function getInfo() {
    $class = static::class;
    return [
      '#input' => TRUE,
      '#process' => [
        [$class, 'processPlugin'],
      ],
      '#theme_wrappers' => ['container'],
    ];
  }

  /**
   * Expands the element.
   */
  public static function processPlugin(&$element, FormStateInterface $form_state, &$complete_form) {
    $wrapper = ($element['#uuid'] ?? $element['#id']) . '-wrapper';
    $element += [
      '#type' => 'fieldset',
      '#collapsible' => FALSE,
      '#prefix' => '<div id="' . $wrapper . '">',
      '#suffix' => '</div>',
    ];
    $element['#tree'] = TRUE;

    $group_condition_logic = $element['#default_value']['group_condition_logic'] ?? 'and';
    $conditions_groups = $form_state->getValue($element['#parents']) ?: $element['#default_value'];
    $conditions_groups = array_filter($conditions_groups, function($p) {return is_array($p) && isset($p['condition_logic']);});
    foreach ($conditions_groups as $id => $conditions_group) {
      $element[$id] = [
        '#type' => 'details',
        '#open' => TRUE,
        '#title' => t('Conditions group @i', ['@i' => (int) $id]),
        '#weight' => 0,
      ];
      $element[$id]['conditions'] = [
        '#type' => 'plugin_config',
        '#plugin_manager' => 'plugin.manager.condition',
        '#title' => t('Conditions'),
        '#default_value' => $conditions_group['conditions'] ?? [],
        '#uuid' => $id,
      ];
      $element[$id]['condition_logic'] = [
        '#title' => t('Conditions logic'),
        '#type' => 'select',
        '#options' => ['and' => t('And'), 'or' => t('Or')],
        '#default_value' => $conditions_group['condition_logic'],
      ];
      $element[$id]['remove'] = [
        '#type' => 'submit',
        '#value' => t('Remove'),
        '#submit' => [[self::class, 'removeConditionGroup']],
        '#ajax' => [
          'callback' => [self::class, 'removeConditionGroupAjax'],
          'event' => 'click',
          'wrapper' => $wrapper,
        ],
        '#name' => 'remove-plugin-' . $id,
        '#limit_validation_errors' => [array_slice($element['#parents'], 0, -1)],
      ];
    }

    $element['group_condition_logic'] = [
      '#title' => t('Conditions groups logic'),
      '#type' => 'select',
      '#options' => ['and' => t('And'), 'or' => t('Or')],
      '#default_value' => $group_condition_logic,
      '#weight' => 10,
    ];
    $element['add'] = [
      '#type' => 'submit',
      '#value' => t('Add conditions group'),
      '#submit' => [[self::class, 'addConditionsGroup']],
      '#ajax' => [
        'callback' => [self::class, 'addConditionsGroupAjax'],
        'event' => 'click',
        'wrapper' => $wrapper,
      ],
      '#name' => 'add-conditions-group-' . ($element['#uuid'] ?? 'no'),
      '#weight' => 20,
      '#limit_validation_errors' => [array_slice($element['#parents'], 0, -1)],
    ];

    $form_state->addCleanValueKey(array_merge($element['#array_parents'], ['add']));

    return $element;
  }

  public static function addConditionsGroup(&$form, FormStateInterface $form_state) {
    $triggering_element = $form_state->getTriggeringElement();
    $parents = array_slice($triggering_element['#parents'], 0, -1);
    $plugins = $form_state->getValue($parents);
    $plugins[] = [
      'condition_logic' => 'and',
    ];
    $form_state->setValue($parents, $plugins);

    $form_state->setRebuild();
  }

  public static function addConditionsGroupAjax(array &$form, FormStateInterface $form_state) {
    $button = $form_state->getTriggeringElement();
    $parents = array_slice($button['#parents'], 0 , -1);
    $element = NestedArray::getValue($form, $parents) ?? [];
    return $element;
  }

  public static function removeConditionGroup(array &$form, FormStateInterface $form_state) {
    $triggering_element = $form_state->getTriggeringElement();
    $parents = array_slice($triggering_element['#parents'], 0, -2);
    $ids = array_slice($triggering_element['#parents'], -2, -1);
    $plugins = $form_state->getValue($parents);
    unset($plugins[reset($ids)]);
    $form_state->setValue($parents, $plugins);

    $form_state->setRebuild();
  }

  public static function removeConditionGroupAjax(array &$form, FormStateInterface $form_state) {
    $button = $form_state->getTriggeringElement();
    $parents = array_slice($button['#parents'], 0 , -2);
    $element = NestedArray::getValue($form, $parents) ?? [];
    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public static function valueCallback(&$element, $input, FormStateInterface $form_state) {
    if ($input) {
      foreach ($input as $id => $condition_group) {
        if (!is_array($condition_group) || empty($condition_group['condition_logic'])) {
          continue;
        }
        $p_element = [
          '#plugin_manager' => 'plugin.manager.condition',
        ];
        $input[$id]['conditions'] = PluginConfig::valueCallback($p_element, $condition_group['conditions'], $form_state);
      }
    }

    return $input ?? [];
  }

}
